%
% CRFaverageZ
%
% This function averages reflectivity (after converting to linear units)
%
% USAGE:
%    [timeav,heightav,Zav] = CRFaverageZ(time,height,Z,st,sh)
%
% WHERE:
%    st and sh are integers specifying the number of pixels to average
%    in time and height respectively.
%    However, st can be a vector of times onto which the reflectivity field should
%    be averaged.

function [timeav,heightav,Zav] = CRFaverageZ(time,height,Z,st,sh)

[nh,nt] = size(Z);
timeav = time;
heightav = height;

if length(st)==1

nh = fix(nh/sh)*sh;
nt = fix(nt/st)*st;
linZ = 10.^(Z(1:nh,1:nt)./10);

size(linZ);

A = eye(nt/st);
if st > 1
   aa = [1 zeros(1,st-1)];
   aaa = aa([ones(1,nt/st)],:)';
   aaaa = cumsum(aaa(:));
   A = A(aaaa,:)./st;
   timeav = mean(reshape(time(1:nt),st,nt/st));
end

B = eye(nh/sh);
if sh > 1
   bb = [1 zeros(1,sh-1)];
   bbb = bb([ones(1,nh/sh)],:)';
   bbbb = cumsum(bbb(:));
   B = B(bbbb,:)'./sh;
   heightav = mean(reshape(height(1:nh),sh,nh/sh));
end

Zav = 10.*log10(B*(linZ*A));

else

timeav=st;
mt=length(timeav);
it=1:mt;
Ztemp=NaN*ones(nh,mt);
indext=round(fastllup(timeav,it,time));
for ii=it
   index=find(indext==ii);
         if length(index) == 1
            Ztemp(:,ii) = Z(:,index);
         elseif length(index) == 0
            Ztemp(:,ii) = NaN*ones(nh,1);
         else
            Ztemp(:,ii) = 10.*log10(mean(10.^(0.1*Z(:,index)')))';
         end
end

if sh > 1
   nh = fix(nh/sh)*sh;
   B = eye(nh/sh);
   bb = [1 zeros(1,sh-1)];
   bbb = bb([ones(1,nh/sh)],:)';
   bbbb = cumsum(bbb(:));
   B = B(bbbb,:)'./sh;
   heightav = mean(reshape(height(1:nh),sh,nh/sh));
   Zav = 10.*log10(B*(10.^(Ztemp(1:nh,:)./10)));
else
   Zav=Ztemp;
end


end



