function [z2] = smoothft2(x,y,z,smx,smy)

%smoothft2(x,y,z,smx,smy): Smooths a matrix in two dimensions
% Use: z2=smoothft2(x,y,z,smx,smy)
% where x, y are vectors or matrices
% z is the matrix containg the function to be smoothed
% smx, smy are the cut-off wavelengths in the x and y directions

[ny nx]=size(z);
lx=max(max(x))-min(min(x));
ly=max(max(y))-min(min(y));

zt=fft2(z);

xm=nx/2;
ym=ny/2;
disx=ones(ny,nx);disy=disx;

for i=2:nx
  for j=2:ny
    if i>xm
      wlx=lx/(nx-i+1);
    else
      wlx=lx/(i-1);
    end
    if j>ym
      wly=ly/(ny-j+1);
    else
      wly=ly/(j-1);
    end
    disx(j,i)=wlx>smx;
    disy(j,i)=wly>smy;
  end
end

zt=zt.*disx.*disy;

z2=real(ifft2(zt));
