function [coefft,exponent,f,Zfft,Zfit,Fave,Zave,Zsample] = spectrum2(frequency,data)
% This function plots a log-log frequency spectrum.
% SYNTAX: spectra(frequency,data);
% where 'frequency' is a single value in Hertz, and 'data' is a
%  vector of data values sampled at the frequency specified.

Zlf = data;
index = find(finite(Zlf));
Zlf = Zlf(index);

si = size(Zlf,2);
sn=1;
n = 2.^(floor(log(si)/log(2)));
Zlf=Zlf(1:n);
disp(['NUMBER OF POINTS USED:  ' num2str(n) ]);

%size(Zlf);

%detrend the data
fit=polyfit([1:length(Zlf)],Zlf,1);Zlf=Zlf-[1:length(Zlf)]*fit(1)-fit(2);

Y = fft(Zlf,n);
EnergyDensity = Y.* conj(Y)./(n.^2 * (frequency/n));
Zfft = EnergyDensity(2:n./2+1);
Zfft = (Zfft+[EnergyDensity(n:-1:n./2+2) 0]);

Zfftsam = 2*[Zfft(1:n/2-1)./2 Zfft(n./2)];
Zfftsam=sqrt(Zfftsam.*frequency./(n)).*exp(i*2*pi*rand(size(Zfft))).*n;
Zfftsam=[0 Zfftsam(1:n./2) conj(Zfftsam(n./2:-1:2))];
Zsample=real(ifft(Zfftsam));


disp(['MAXIMUM:  ' num2str(max(Zfft))]);

f = frequency*(1:n./2)/n;
disp(['FREQUENCY INTERVAL:  ' num2str(frequency./n)]);
logf = log(f);
dlogg = logf(2) - logf(1);
logg = logf(1):dlogg:logf(n./2);
g = exp(logg);
sg = size(logg,2);
Zg = zeros(sn,sg);
for gg = 1:sg
   a = find((logf > (logg(gg)-dlogg)) & (logf < (logg(gg)+dlogg)));
   if max(size(a)) == 1
      Zg(:,gg) = Zfft(a);
   else
      Zg(:,gg) = mean(Zfft(a)')';
   end
end
fZ = (ones(sn,1)*f).*Zfft;
gZ = (ones(sn,1)*g).*Zg;

nminfit=5;

Eave=[];
rpt=1;
numb=floor(n./14);
Mave=zeros(n/2,numb);
for ii=1:numb
%size(Mave([n/2+(ii-numb-1)*7:7:n/2+(ii-numb)*7],ii))
   Mave([n/2+(ii-numb-1)*7+1:n/2+(ii-numb)*7],ii)=ones(7,1)./7;
end

%size(f)
%size(Zfft)
%size(Mave)
Fave=f*Mave;
Zave=Zfft*Mave;   


%clf
%note - I've multiplied these by f to get energy density...
set(gcf,'DefaultAxesFontSize',12);
loglog(f,f'.*Zfft','k','LineWidth',0.2)
hold on
loglog(Fave,Fave.*Zave,'b','LineWidth',0.2)


fit=polyfit(log(Fave),log(Zave),1);
disp(['FIT AFTER 7pt AVERAGING (--):  E(f) = ' num2str(exp(fit(2))) ' f^' num2str(fit(1))])
hold on;plot(f,exp(polyval(fit,log(f))),'k-','LineWidth',1);


coefft = exp(fit(2));
exponent = fit(1);

Zfit=coefft*f.^exponent;



%fit=polyfit(log(g(nminfit-1:length(g))),log(Zg(nminfit-1:length(g))),1);
%disp(['BEST UNWEIGHTED FIT (-.):     E(f) = ' num2str(exp(fit(2))) ' f^' num2str(fit(1))])
%hold on;plot(g,exp(polyval(fit,log(g))),'k-','LineWidth',1);
%disp(['MAX VALUE:  ' num2str(max(Zfft))]);

hold on

%loglog(g,Zg','g')



xlabel('Wavenumber k (m^{-1})')
ylabel('Reflectivity energy density E (dBZ^2 m)')


hold off


%hold on
%plot(log(f),polyval(fit,log(f)),'w')
