function [vel]=zap3(v,maxvar,lonely1,lonely2)

%zap3(v,maxvar,lonely1,lonely2): Removes regions of high variance, checking for
% 'lonely' points in both data rich and data sparce areas
% Use: vel=zap3(v,maxvar,lonely1,lonely2)
% where v is the matrix to be zapped
% vel is the zapped matrix (of the same size etc as v)
% maxvar is the maximum variance allowed, calculated from the surrounding
% points.
% lonely1: if the number of NaNs surrounding an unremoved data point is greater
% that lonely1, then that data point is removed (to prevent spurious points in 
% regions of no real data). lonely1=8 has no effect
% lonely2: if the number of NaNs surrounding a removed point is less
% than lonely2, then that removed point is replaced (to prevent holes
% in the data). lonely2=0 has no effect
% The folding of velocities (assumed = 30) is taken into account
% by evaluating the variance twice, the second time having shifted the 
% velocities by 15m/s. 
% Maxvar=5,lonely1=4,lonely2=4 seems to work quite well for rhi's
% Maxvar=6,lonely1=4,lonely2=4 seems to work quite well for ppi's

[mm nn]=size(v);
mv2=maxvar^2;

%work out the variance matrix
var=0;
var(2:mm-1,2:nn-1)=((v(1:mm-2,1:nn-2)-v(2:mm-1,2:nn-1)).^2)+ ...
                   ((v(2:mm-1,1:nn-2)-v(2:mm-1,2:nn-1)).^2)+ ...
                   ((v(3:mm,1:nn-2)-v(2:mm-1,2:nn-1)).^2)+ ...
                   ((v(1:mm-2,2:nn-1)-v(2:mm-1,2:nn-1)).^2)+ ...
                   ((v(3:mm,2:nn-1)-v(2:mm-1,2:nn-1)).^2)+ ...
                   ((v(1:mm-2,3:nn)-v(2:mm-1,2:nn-1)).^2)+ ...
                   ((v(2:mm-1,3:nn)-v(2:mm-1,2:nn-1)).^2)+ ...
                   ((v(3:mm,1:nn-2)-v(2:mm-1,2:nn-1)).^2);
var(2:mm-1,2:nn-1)=var(2:mm-1,2:nn-1)/8;

var(2:mm-1,nn)=((v(1:mm-2,nn)-v(2:mm-1,nn)).^2)+ ...
               ((v(1:mm-2,nn-1)-v(2:mm-1,nn)).^2)+ ...
               ((v(2:mm-1,nn-1)-v(2:mm-1,nn)).^2)+ ...
               ((v(3:mm,nn-1)-v(2:mm-1,nn)).^2)+ ...
               ((v(3:mm,nn)-v(2:mm-1,nn)).^2);
var(2:mm-1,nn)=var(2:mm-1,nn)/5;

var(2:mm-1,1)=((v(1:mm-2,1)-v(2:mm-1,1)).^2)+ ...
              ((v(1:mm-2,2)-v(2:mm-1,1)).^2)+ ...
              ((v(2:mm-1,2)-v(2:mm-1,1)).^2)+ ...
              ((v(3:mm,2)-v(2:mm-1,1)).^2)+ ...
              ((v(3:mm,1)-v(2:mm-1,1)).^2);
var(2:mm-1,1)=var(2:mm-1,1)/5;

var(1,2:nn-1)=((v(1,1:nn-2)-v(1,2:nn-1)).^2)+ ...
              ((v(2,1:nn-2)-v(1,2:nn-1)).^2)+ ...
              ((v(2,2:nn-1)-v(1,2:nn-1)).^2)+ ...
              ((v(2,3:nn)-v(1,2:nn-1)).^2)+ ...
              ((v(1,3:nn)-v(1,2:nn-1)).^2);
var(1,2:nn-1)=var(1,2:nn-1)/5;

var(mm,2:nn-1)=((v(mm,1:nn-2)-v(mm,2:nn-1)).^2)+ ...
               ((v(mm-1,1:nn-2)-v(mm,2:nn-1)).^2)+ ...
               ((v(mm-1,2:nn-1)-v(mm,2:nn-1)).^2)+ ...
               ((v(mm-1,3:nn)-v(mm,2:nn-1)).^2)+ ...
               ((v(mm,3:nn)-v(mm,2:nn-1)).^2);

var(mm,2:nn-1)=var(mm,2:nn-1)/5;

%summarize the above in a mask

mask1=(var<mv2);

%now 'rotate' v by 15 m/s (storing original v as v1)
v1=v;

v=v+15;
[i j]=find(v>15);
for k=1:length(i)
 v(i(k),j(k))=v(i(k),j(k))-30;
end 

%do the variance stuff again
var=0;
var(2:mm-1,2:nn-1)=((v(1:mm-2,1:nn-2)-v(2:mm-1,2:nn-1)).^2)+ ...
                   ((v(2:mm-1,1:nn-2)-v(2:mm-1,2:nn-1)).^2)+ ...
                   ((v(3:mm,1:nn-2)-v(2:mm-1,2:nn-1)).^2)+ ...
                   ((v(1:mm-2,2:nn-1)-v(2:mm-1,2:nn-1)).^2)+ ...
                   ((v(3:mm,2:nn-1)-v(2:mm-1,2:nn-1)).^2)+ ...
                   ((v(1:mm-2,3:nn)-v(2:mm-1,2:nn-1)).^2)+ ...
                   ((v(2:mm-1,3:nn)-v(2:mm-1,2:nn-1)).^2)+ ...
                   ((v(3:mm,1:nn-2)-v(2:mm-1,2:nn-1)).^2);
var(2:mm-1,2:nn-1)=var(2:mm-1,2:nn-1)/8;

var(2:mm-1,nn)=((v(1:mm-2,nn)-v(2:mm-1,nn)).^2)+ ...
               ((v(1:mm-2,nn-1)-v(2:mm-1,nn)).^2)+ ...
               ((v(2:mm-1,nn-1)-v(2:mm-1,nn)).^2)+ ...
               ((v(3:mm,nn-1)-v(2:mm-1,nn)).^2)+ ...
               ((v(3:mm,nn)-v(2:mm-1,nn)).^2);
var(2:mm-1,nn)=var(2:mm-1,nn)/5;

var(2:mm-1,1)=((v(1:mm-2,1)-v(2:mm-1,1)).^2)+ ...
              ((v(1:mm-2,2)-v(2:mm-1,1)).^2)+ ...
              ((v(2:mm-1,2)-v(2:mm-1,1)).^2)+ ...
              ((v(3:mm,2)-v(2:mm-1,1)).^2)+ ...
              ((v(3:mm,1)-v(2:mm-1,1)).^2);
var(2:mm-1,1)=var(2:mm-1,1)/5;

var(1,2:nn-1)=((v(1,1:nn-2)-v(1,2:nn-1)).^2)+ ...
              ((v(2,1:nn-2)-v(1,2:nn-1)).^2)+ ...
              ((v(2,2:nn-1)-v(1,2:nn-1)).^2)+ ...
              ((v(2,3:nn)-v(1,2:nn-1)).^2)+ ...
              ((v(1,3:nn)-v(1,2:nn-1)).^2);
var(1,2:nn-1)=var(1,2:nn-1)/5;

var(mm,2:nn-1)=((v(mm,1:nn-2)-v(mm,2:nn-1)).^2)+ ...
               ((v(mm-1,1:nn-2)-v(mm,2:nn-1)).^2)+ ...
               ((v(mm-1,2:nn-1)-v(mm,2:nn-1)).^2)+ ...
               ((v(mm-1,3:nn)-v(mm,2:nn-1)).^2)+ ...
               ((v(mm,3:nn)-v(mm,2:nn-1)).^2);

var(mm,2:nn-1)=var(mm,2:nn-1)/5;

%summarize the above in a mask

mask2=(var<mv2);

%combine the two masks, requiring a discontinuity in both velocity fields

mask=((mask1==1)|(mask2==1));

%now apply to v1

vel=NaN*ones(mm,nn);
index=find(mask==1);
vel(index)=v1(index);

%check for lonely points in data sparse regions
mask3=0;
i=2:mm-1;
j=2:nn-1;
mask3(i,j)=(isnan(vel(i,j))==0)&((isnan(vel(i-1,j))+isnan(vel(i-1,j+1))+isnan(vel(i,j+1))+isnan(vel(i+1,j+1))+isnan(vel(i+1,j))+isnan(vel(i+1,j-1))+isnan(vel(i,j-1))+isnan(vel(i-1,j-1)))>lonely1);
i=1;
j=2:nn-1;
mask3(i,j)=(isnan(vel(i,j))==0)&((isnan(vel(i,j+1))+isnan(vel(i+1,j+1))+isnan(vel(i+1,j))+isnan(vel(i+1,j-1))+isnan(vel(i,j-1)))>(lonely1*5/8));
i=mm;
j=2:nn-1;
mask3(i,j)=(isnan(vel(i,j))==0)&((isnan(vel(i-1,j))+isnan(vel(i-1,j+1))+isnan(vel(i,j+1))+isnan(vel(i,j-1))+isnan(vel(i-1,j-1)))>(lonely1*5/8));
i=2:mm-1;
j=1;
mask3(i,j)=(isnan(vel(i,j))==0)&((isnan(vel(i-1,j))+isnan(vel(i-1,j+1))+isnan(vel(i,j+1))+isnan(vel(i+1,j+1))+isnan(vel(i+1,j)))>(lonely1*5/8));
i=2:mm-1;
j=nn;
mask3(i,j)=(isnan(vel(i,j))==0)&((isnan(vel(i-1,j))+isnan(vel(i+1,j))+isnan(vel(i+1,j-1))+isnan(vel(i,j-1))+isnan(vel(i-1,j-1)))>(lonely1*5/8));

index=find(mask3==1);
vel(index)=NaN*ones(size(index));

%check for lonely NaNs in data rich regions
mask4=0;
i=2:mm-1;
j=2:nn-1;
mask4(i,j)=(isnan(vel(i,j)))&((isnan(vel(i-1,j))+isnan(vel(i-1,j+1))+isnan(vel(i,j+1))+isnan(vel(i+1,j+1))+isnan(vel(i+1,j))+isnan(vel(i+1,j-1))+isnan(vel(i,j-1))+isnan(vel(i-1,j-1)))<lonely2);
i=1;
j=2:nn-1;
mask4(i,j)=(isnan(vel(i,j)))&((isnan(vel(i,j+1))+isnan(vel(i+1,j+1))+isnan(vel(i+1,j))+isnan(vel(i+1,j-1))+isnan(vel(i,j-1)))<(lonely2*5/8));
i=mm;
j=2:nn-1;
mask4(i,j)=(isnan(vel(i,j)))&((isnan(vel(i-1,j))+isnan(vel(i-1,j+1))+isnan(vel(i,j+1))+isnan(vel(i,j-1))+isnan(vel(i-1,j-1)))<(lonely2*5/8));
i=2:mm-1;
j=1;
mask4(i,j)=(isnan(vel(i,j)))&((isnan(vel(i-1,j))+isnan(vel(i-1,j+1))+isnan(vel(i,j+1))+isnan(vel(i+1,j+1))+isnan(vel(i+1,j)))<(lonely2*5/8));
i=2:mm-1;
j=nn;
mask4(i,j)=(isnan(vel(i,j)))&((isnan(vel(i-1,j))+isnan(vel(i+1,j))+isnan(vel(i+1,j-1))+isnan(vel(i,j-1))+isnan(vel(i-1,j-1)))<(lonely2*5/8));

index=find(mask4==1);
vel(index)=v1(index);








